package logging

import (
	`github.com/google/uuid`
)

/*
	AddDefaultLogger adds a default Logger (as would be determined by GetLogger) to a MultiLogger.

	identifier is a string to use to identify the added Logger in MultiLogger.Loggers.
	If empty, one will be automatically generated.

	A pointer to a WinEventID struct may be specified for eventIDs to map extended logging levels (as Windows only supports three levels natively).
	If it is nil, a default one (DefaultEventID) will be used.

	logPaths is an (optional) list of strings to use as paths to test for writing. If the file can be created/written to,
	it will be used (assuming you have no higher-level loggers available).

	Only the first logPaths entry that "works" will be used, later entries will be ignored.
	Currently this will almost always return a WinLogger.
*/
func (m *MultiLogger) AddDefaultLogger(identifier string, eventIDs *WinEventID, logPaths ...string) (err error) {

	var l Logger
	var exists bool

	if identifier == "" {
		identifier = uuid.New().String()
	}

	if _, exists = m.Loggers[identifier]; exists {
		err = ErrExistingLogger
		return
	}

	if logPaths != nil {
		l, err = GetLogger(m.EnableDebug, m.Prefix, eventIDs, logPaths...);
	} else {
		l, err = GetLogger(m.EnableDebug, m.Prefix, eventIDs);
	}
	if err != nil {
		return
	}

	m.Loggers[identifier] = l

	return
}

/*
	AddWinLogger adds a WinLogger to a MultiLogger. Note that this is a VERY generalized interface to the Windows Event Log.

	If you require more robust logging capabilities (e.g. custom event IDs per uniquely identifiable event),
	you will want to set up your own logger (golang.org/x/sys/windows/svc/eventlog).

	identifier is a string to use to identify the added WinLogger in MultiLogger.Loggers.
	If empty, one will be automatically generated.

	A blank source will return an error as it's used as the source name. Other functions, struct fields, etc. will refer to this as the "prefix".

	A pointer to a WinEventID struct may be specified for eventIDs to map extended logging levels (as Windows only supports three levels natively).
	If it is nil, a default one (DefaultEventID) will be used.

	See GetLogger for details.
*/
func (m *MultiLogger) AddWinLogger(identifier, source string, eventIDs *WinEventID) (err error) {

	var exists bool

	if identifier == "" {
		identifier = uuid.New().String()
	}

	if _, exists = m.Loggers[identifier]; exists {
		err = ErrExistingLogger
		return
	}

	if eventIDs == nil {
		eventIDs = DefaultEventID
	}

	m.Loggers[identifier] = &WinLogger{
		Prefix:      source,
		EnableDebug: m.EnableDebug,
		eids:        eventIDs,
	}
	m.Loggers[identifier].Setup()

	m.Loggers[identifier].Info("logger initialized of type %T with prefix %v", m.Loggers[identifier], m.Loggers[identifier].GetPrefix())

	return
}
